@@ -18,10 +18,8 @@ import logging
 import math
 import os
 import re
-import requests
 import shlex
 import shutil
-import six
 import stat
 import tempfile
 import time
@@ -32,11 +30,10 @@ from oslo_serialization import base64
 from oslo_utils import excutils
 from oslo_utils import imageutils
 from oslo_utils import units
+import requests
+import six
 
 from ironic_lib.common.i18n import _
-from ironic_lib.common.i18n import _LE
-from ironic_lib.common.i18n import _LI
-from ironic_lib.common.i18n import _LW
 from ironic_lib import disk_partitioner
 from ironic_lib import exception
 from ironic_lib import utils
@@ -97,9 +94,9 @@ def list_partitions(device):
     for line in lines:
         match = _PARTED_PRINT_RE.match(line)
         if match is None:
-            LOG.warning(_LW("Partition information from parted for device "
-                            "%(device)s does not match "
-                            "expected format: %(line)s"),
+            LOG.warning("Partition information from parted for device "
+                        "%(device)s does not match "
+                        "expected format: %(line)s",
                         dict(device=device, line=line))
             continue
         # Cast int fields to ints (some are floats and we round them down)
@@ -370,8 +367,8 @@ def destroy_disk_metadata(dev, node_uuid):
                               run_as_root=True,
                               use_standard_locale=True)
 
-    LOG.info(_LI("Disk metadata on %(dev)s successfully destroyed for node "
-                 "%(node)s"), {'dev': dev, 'node': node_uuid})
+    LOG.info("Disk metadata on %(dev)s successfully destroyed for node "
+             "%(node)s", {'dev': dev, 'node': node_uuid})
 
 
 def _get_configdrive(configdrive, node_uuid, tempdir=None):
@@ -489,8 +486,8 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
                                     boot_option=boot_option,
                                     boot_mode=boot_mode,
                                     disk_label=disk_label)
-        LOG.info(_LI("Successfully completed the disk device"
-                     " %(dev)s partitioning for node %(node)s"),
+        LOG.info("Successfully completed the disk device"
+                 " %(dev)s partitioning for node %(node)s",
                  {'dev': dev, "node": node_uuid})
 
         ephemeral_part = part_dict.get('ephemeral')
@@ -522,8 +519,8 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
         if configdrive_part:
             # Copy the configdrive content to the configdrive partition
             dd(configdrive_file, configdrive_part)
-            LOG.info(_LI("Configdrive for node %(node)s successfully copied "
-                         "onto partition %(partition)s"),
+            LOG.info("Configdrive for node %(node)s successfully copied "
+                     "onto partition %(partition)s",
                      {'node': node_uuid, 'partition': configdrive_part})
 
     finally:
@@ -533,20 +530,20 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
             utils.unlink_without_raise(configdrive_file)
 
     populate_image(image_path, root_part)
-    LOG.info(_LI("Image for %(node)s successfully populated"),
+    LOG.info("Image for %(node)s successfully populated",
              {'node': node_uuid})
 
     if swap_part:
         utils.mkfs(fs='swap', path=swap_part, label='swap1')
-        LOG.info(_LI("Swap partition %(swap)s successfully formatted "
-                     "for node %(node)s"),
+        LOG.info("Swap partition %(swap)s successfully formatted "
+                 "for node %(node)s",
                  {'swap': swap_part, 'node': node_uuid})
 
     if ephemeral_part and not preserve_ephemeral:
         utils.mkfs(fs=ephemeral_format, path=ephemeral_part,
                    label="ephemeral0")
-        LOG.info(_LI("Ephemeral partition %(ephemeral)s successfully "
-                     "formatted for node %(node)s"),
+        LOG.info("Ephemeral partition %(ephemeral)s successfully "
+                 "formatted for node %(node)s",
                  {'ephemeral': ephemeral_part, 'node': node_uuid})
 
     uuids_to_return = {
@@ -561,7 +558,7 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
 
     except processutils.ProcessExecutionError:
         with excutils.save_and_reraise_exception():
-            LOG.error(_LE("Failed to detect %s"), part)
+            LOG.error("Failed to detect %s", part)
 
     return uuids_to_return
 
@@ -631,7 +628,7 @@ def _get_labelled_partition(device_path, label, node_uuid):
                    for v in shlex.split(device))}
             if not dev:
                 continue
-            if dev['LABEL'].upper() == label.upper():
+            if dev['LABEL'] == label:
                 if found_part:
                     found_2 = '/dev/%(part)s' % {'part': dev['NAME'].strip()}
                     found = [found_part, found_2]
@@ -696,28 +693,6 @@ def _fix_gpt_structs(device, node_uuid):
         raise exception.InstanceDeployFailure(msg)
 
 
-def _fix_gpt_partition(device, node_uuid):
-    """Fix GPT partition
-
-    Fix GPT table information when image is written to a disk which
-    has a bigger extend (e.g. 30GB image written on a 60Gb physical disk).
-
-    :param device: The device path.
-    :param node_uuid: UUID of the Node.
-    :raises: InstanceDeployFailure if exception is caught.
-    """
-    try:
-        disk_is_gpt_partitioned = _is_disk_gpt_partitioned(device, node_uuid)
-        if disk_is_gpt_partitioned:
-            _fix_gpt_structs(device, node_uuid)
-    except Exception as e:
-        msg = (_('Failed to fix GPT partition on disk %(disk)s '
-                 'for node %(node)s. Error: %(error)s') %
-               {'disk': device, 'node': node_uuid, 'error': e})
-        LOG.error(msg)
-        raise exception.InstanceDeployFailure(msg)
-
-
 def create_config_drive_partition(node_uuid, device, configdrive):
     """Create a partition for config drive
 
@@ -750,7 +725,6 @@ def create_config_drive_partition(node_uuid, device, configdrive):
                   "device: %(dev)s for node %(node)s",
                   {'dev': device, 'size': confdrive_mb, 'node': node_uuid})
 
-        _fix_gpt_partition(device, node_uuid)
         if config_drive_part:
             LOG.debug("Configdrive for node %(node)s exists at "
                       "%(part)s",
@@ -759,6 +733,7 @@ def create_config_drive_partition(node_uuid, device, configdrive):
             cur_parts = set(part['number'] for part in list_partitions(device))
 
             if _is_disk_gpt_partitioned(device, node_uuid):
+                _fix_gpt_structs(device, node_uuid)
                 create_option = '0:-%dMB:0' % MAX_CONFIG_DRIVE_SIZE_MB
                 utils.execute('sgdisk', '-n', create_option, device,
                               run_as_root=True)
@@ -789,9 +764,9 @@ def create_config_drive_partition(node_uuid, device, configdrive):
                 endlimit = '-0'
                 if _is_disk_larger_than_max_size(device, node_uuid):
                     # Need to create a small partition at 2TB limit
-                    LOG.warning(_LW("Disk size is larger than 2TB for "
-                                    "node %(node)s. Creating config drive "
-                                    "at the end of the disk %(disk)s."),
+                    LOG.warning("Disk size is larger than 2TB for "
+                                "node %(node)s. Creating config drive "
+                                "at the end of the disk %(disk)s.",
                                 {'node': node_uuid, 'disk': device})
                     startlimit = (MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR -
                                   MAX_CONFIG_DRIVE_SIZE_MB - 1)
@@ -814,17 +789,23 @@ def create_config_drive_partition(node_uuid, device, configdrive):
             else:
                 config_drive_part = '%s%s' % (device, new_part.pop())
 
-            # NOTE(vdrok): the partition was created successfully, let's wait
-            # for it to appear in /dev.
+            LOG.debug('Waiting until udev event queue is empty')
+            utils.execute('udevadm', 'settle')
+
+            # NOTE(vsaienko): check that devise actually exists,
+            # it is not handled by udevadm when using ISCSI, for more info see:
+            # https://bugs.launchpad.net/ironic/+bug/1673731
+            # Do not use 'udevadm settle --exit-if-exist' here
             LOG.debug('Waiting for the config drive partition %(part)s '
                       'on node %(node)s to be ready for writing.',
                       {'part': config_drive_part, 'node': node_uuid})
-            utils.execute('udevadm', 'settle',
-                          '--exit-if-exists=%s' % config_drive_part)
+            utils.execute('test', '-e', config_drive_part,
+                          check_exit_code=[0], attempts=15,
+                          delay_on_retry=True)
 
         dd(confdrive_file, config_drive_part)
-        LOG.info(_LI("Configdrive for node %(node)s successfully "
-                     "copied onto partition %(part)s"),
+        LOG.info("Configdrive for node %(node)s successfully "
+                 "copied onto partition %(part)s",
                  {'node': node_uuid, 'part': config_drive_part})
 
     except (processutils.UnknownArgumentError,
