@@ -14,12 +14,12 @@
 #    under the License.
 
 import gzip
-import mock
 import os
 import shutil
 import stat
 import tempfile
 
+import mock
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_serialization import base64
@@ -700,7 +700,8 @@ class OtherFunctionTestCase(test_base.BaseTestCase):
         disk_utils.qemu_img_info('img')
         path_exists_mock.assert_called_once_with('img')
         execute_mock.assert_called_once_with('env', 'LC_ALL=C', 'LANG=C',
-                                             'qemu-img', 'info', 'img')
+                                             'qemu-img', 'info', 'img',
+                                             prlimit=mock.ANY)
         qemu_img_info_mock.assert_called_once_with('out')
 
     @mock.patch.object(utils, 'execute', autospec=True)
@@ -708,7 +709,8 @@ class OtherFunctionTestCase(test_base.BaseTestCase):
         disk_utils.convert_image('source', 'dest', 'out_format')
         execute_mock.assert_called_once_with('qemu-img', 'convert', '-O',
                                              'out_format', 'source', 'dest',
-                                             run_as_root=False)
+                                             run_as_root=False,
+                                             prlimit=mock.ANY)
 
     @mock.patch.object(os.path, 'getsize', autospec=True)
     @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
@@ -1045,9 +1047,11 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         mock_execute.assert_has_calls([
             mock.call('sgdisk', '-n', '0:-64MB:0', self.dev,
                       run_as_root=True),
-            mock.call('udevadm', 'settle',
-                      '--exit-if-exists=%s' % expected_part),
+            mock.call('udevadm', 'settle'),
+            mock.call('test', '-e', expected_part, attempts=15,
+                      check_exit_code=[0], delay_on_retry=True)
         ])
+
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_fix_gpt.assert_called_with(self.dev, self.node_uuid)
@@ -1056,7 +1060,7 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
 
     @mock.patch.object(disk_utils, 'count_mbr_partitions', autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
-    @mock.patch.object(disk_utils.LOG, 'warning')
+    @mock.patch.object(disk_utils.LOG, 'warning', autospec=True)
     @mock.patch.object(utils, 'unlink_without_raise',
                        autospec=True)
     @mock.patch.object(disk_utils, 'dd',
@@ -1135,8 +1139,9 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                     '-0', run_as_root=True)
         mock_execute.assert_has_calls([
             parted_call,
-            mock.call('udevadm', 'settle',
-                      '--exit-if-exists=%s' % expected_part),
+            mock.call('udevadm', 'settle'),
+            mock.call('test', '-e', expected_part, attempts=15,
+                      check_exit_code=[0], delay_on_retry=True)
         ])
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
